import datetime
import json

from django.contrib import messages
from django.shortcuts import reverse, redirect, get_object_or_404
from django.http.response import HttpResponse
# from django.contrib.auth.decorators import user_passes_test
# from django.contrib.auth.mixins import LoginRequiredMixin
# from django.utils.decorators import method_decorator
from django.views.generic import CreateView, TemplateView, ListView, UpdateView

from booking.models import Booking, BookingManager
from booking.forms import BookingForm, BookingManagerForm

# # # # # # #
# Admin Part
# # # # # # #


class AdminHomeView(TemplateView):
    model = Booking
    template_name = "booking/admin/dashboard.html"

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context["last_bookings"] = Booking.objects.filter().order_by(
            "date", "time")[:10]
        context["waiting_bookings"] = Booking.objects.filter(
            approved=False).order_by("-date", "time")[:10]
        return context


class BookingListView(ListView):
    form_class = Booking
    template_name = "booking/admin/appointment_list.html"

    def get_queryset(self):
        return Booking.objects.filter()[:20]


# @method_decorator(user_passes_test(lambda u: u.is_superuser), name='dispatch')
class BookingSettingsView(UpdateView):
    form_class = BookingManagerForm
    template_name = "booking/admin/appointment_settings.html"

    def get_object(self):
        obj = BookingManager.objects.filter()
        if obj.exists():
            return obj[0]
        else:
            return BookingManager.objects.create(start_time="09:00", end_time="17:00")

    def get_success_url(self):
        return reverse("booking_settings") + "?type=successed"


def bookingUpdateView(request, id, type):
    if request.method == "GET":
        item = get_object_or_404(Booking, id=id)
        if type == "delete":
            item.delete()
            messages.warning(request, "The item successfully deleted!")
        elif type == "approved":
            item.approved = True
            item.save()
            messages.success(request, "The item successfully approved!")

        return redirect(reverse("booking_list"))

    return redirect(reverse("create_booking"))


# # # # # # # #
# Booking Part
# # # # # # # #

def add_delta(tme, delta):
    # transform to a full datetime first
    return (datetime.datetime.combine(
        datetime.date.today(), tme
    ) + delta).time()


class BookingCreateView(CreateView):
    form_class = BookingForm
    template_name = "booking/user/create_booking.html"

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)

        return context

    def get_success_url(self):
        url = reverse("create_booking")
        return url + "?type=successed"


def get_available_time(request):
    if request.method == 'POST':
        post_data = json.loads(request.body.decode("utf-8"))

        b_manager = BookingManager.objects.first()
        existing_bookings = Booking.objects.filter(
            date=post_data["date"]).values_list('time')
        next_time = b_manager.start_time
        time_list = []
        while True:
            is_taken = any([x[0] == next_time for x in existing_bookings])
            time_list.append(
                {"time": ":".join(str(next_time).split(":")[:-1]), "is_taken": is_taken})
            next_time = add_delta(next_time, datetime.timedelta(
                minutes=int(b_manager.period_of_each_booking)))
            if next_time > b_manager.end_time:
                break
        data = json.dumps({'time_list': time_list})
        return HttpResponse(data, content_type='application/json')



# Generated by Django 3.1.7 on 2021-08-07 22:11

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='Booking',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('date', models.DateField()),
                ('time', models.TimeField()),
                ('user_name', models.CharField(max_length=250)),
                ('user_email', models.EmailField(max_length=254)),
                ('approved', models.BooleanField(default=False)),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('user', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
        ),
    ]



from django.urls import path

from .views import BookingCreateView, AdminHomeView, BookingListView, BookingSettingsView, bookingUpdateView, get_available_time


urlpatterns = [
        path("", BookingCreateView.as_view(), name="create_booking"),
        path("admin", AdminHomeView.as_view(), name="admin_dashboard"),
        path("admin/list", BookingListView.as_view(), name="booking_list"),
        path("admin/settings", BookingSettingsView.as_view(), name="booking_settings"),
        path("<int:id>/<str:type>", bookingUpdateView, name="booking_update"),
        path("get-available-time", get_available_time, name="get_available_time"),
]


from booking.models import Booking, BookingManager
from django import forms

class BookingForm(forms.ModelForm):
    class Meta:
        model = Booking
        fields  = ["time", "date", "user_name", "user_email"]

        
class BookingManagerForm(forms.ModelForm):
    start_time = forms.TimeField(widget=forms.TimeInput(format='%H:%M'))
    end_time = forms.TimeField(widget=forms.TimeInput(format='%H:%M'))

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        # add common css classes to all widgets
        for field in iter(self.fields):
            #get current classes from Meta
            input_type = self.fields[field].widget.input_type
            classes = self.fields[field].widget.attrs.get("class")
            if classes is not None:
                classes += " form-check-input" if input_type == "checkbox" else " form-control"
            else:
                classes = " form-check-input" if input_type == "checkbox" else " form-control"
            self.fields[field].widget.attrs.update({
                'class': classes
            })
    class Meta:
        model = BookingManager
        fields  = "__all__"



import os
from setuptools import find_packages, setup

with open(os.path.join(os.path.dirname(__file__), 'README.rst')) as readme:
    README = readme.read()

# allow setup.py to be run from any path
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))


setup(
    name = 'dj-booking',
    version = '1.0',
    description = 'Django Booking is a complete Django booking system as a package.',
    long_description=README,
    url = 'https://github.com/foad-heidari/django-booking',
    author = 'Foad',
    author_email = 'foad.haydri.1377@gmail.com',
    license = 'MIT',
    maintainer = 'Foad',
    maintainer_email = 'foad.haydri.1377@gmail.com',
    keywords = 'django booking appointment appointment-booking booking-system appointment-system doctor-appointment-booking',
    packages=find_packages(),
    include_package_data=True,

    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Django',
        'Framework :: Django :: 1.11',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3 :: Only',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],
)





# Generated by Django 3.1.7 on 2021-08-08 14:13

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('booking', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='BookingManager',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('booking_enable', models.BooleanField(default=True)),
                ('confirmation_required', models.BooleanField(default=True)),
                ('dissable_weekend', models.BooleanField(default=True)),
                ('available_booking_months', models.IntegerField(default=1)),
                ('max_appointment_per_day', models.IntegerField(blank=True, null=True)),
                ('start_time', models.TimeField()),
                ('end_time', models.TimeField()),
                ('period_of_each_booking', models.CharField(choices=[('5', '5M'), ('10', '10M'), ('15', '15M'), ('20', '20M'), ('25', '25M'), ('30', '30M'), ('35', '35M'), ('40', '40M'), ('45', '45M'), ('60', '1H'), ('75', '1H 15M'), ('90', '1H 30M'), ('105', '1H 45M'), ('120', '2H'), ('150', '2H 30M'), ('180', '3H')], default='30', max_length=3)),
                ('max_appointment_per_time', models.IntegerField(default=1)),
            ],
        ),
    ]



from django.db import models
from django.conf import settings


BOOKING_PERIOD = (
    ("5","5M"),
    ("10","10M"),
    ("15","15M"),
    ("20","20M"),
    ("25","25M"),
    ("30","30M"),
    ("35","35M"),
    ("40","40M"),
    ("45","45M"),
    ("60","1H"),
    ("75","1H 15M"), 
    ("90","1H 30M"), 
    ("105","1H 45M"),
    ("120","2H"),
    ("150","2H 30M"),
    ("180","3H"),
)

class Booking(models.Model):
    user = models.ForeignKey(settings.AUTH_USER_MODEL,
                             on_delete=models.CASCADE, blank=True, null=True)
    date = models.DateField()
    time = models.TimeField()
    user_name = models.CharField(max_length=250)
    user_email = models.EmailField()
    approved = models.BooleanField(default=False)

    created_at = models.DateTimeField(auto_now_add=True)
    updated_at = models.DateTimeField(auto_now=True)


    def __str__(self) -> str:
        return self.user_name or "(No Name)"


class BookingManager(models.Model):
    # General
    booking_enable = models.BooleanField(default=True)
    confirmation_required = models.BooleanField(default=True)
    # Date
    dissable_weekend = models.BooleanField(default=True)
    available_booking_months = models.IntegerField(default=1, help_text="if 2, user can only book appointment for next two months.")
    max_appointment_per_day = models.IntegerField(null=True, blank=True)
    # Time
    start_time = models.TimeField()
    end_time = models.TimeField()
    period_of_each_booking = models.CharField(max_length=3, default="30", choices=BOOKING_PERIOD, help_text="How long each appointment take.")
    max_appointment_per_time = models.IntegerField(default=1, help_text="how much appointment can be book for each time.")
